/*
** tlscommon.c for tlsinject in
**
** Made by Averous Julien-Pierre
** E-Mail   <j.averous@sourcemac.com>
**
** Started on  Wed Nov 26 15:47:33 2008 Averous Julien-Pierre
** Last update Sat Dec  6 19:34:20 2008 Averous Julien-Pierre
*/

/*
** Content inspired from the OpenSSL source sample
*/

#include <stdlib.h>
#include <string.h>

#include "tlscommon.h"

/*
** Global variables
*/
static BIO	*bio_err = NULL;
static char	*pass = NULL;

/*
** Global define
*/
#define CA_LIST "/openssl_files/root.pem"

// Handle sigpipe to prevent exit
static void sigpipe_handle(int x)
{
}

// Give password
static int password_cb(char *buf, int num, int rwflag, void *userdata)
{
  if (!pass)
    return 0;

  if(num < strlen(pass) + 1)
    return 0;

  strcpy(buf, pass);
  return strlen(pass);
}

/*!
** Build a context with a certificate and it's passphrase, and a SSL method
**
** @param keyfile The keyfile (pem)
** @param password The password of the keyfile
** @param meth The method to use for the SSL (SSL/DTLS/... see tlsinject.c for example)
**
** @return An SSL context initialized
*/
SSL_CTX *initialize_ctx(char *keyfile, char *password, SSL_METHOD *meth)
{
  SSL_CTX	*ctx;

  if (!bio_err)
  {
    // Global system initialization
    SSL_library_init();
    SSL_load_error_strings();

    // An error write context
    bio_err = BIO_new_fp(stderr, BIO_NOCLOSE);
  }

  // Set up a SIGPIPE handler
  signal(SIGPIPE, sigpipe_handle);

  // Create our context
  ctx = SSL_CTX_new(meth);

  // Load our keys and certificates
  if(!(SSL_CTX_use_certificate_chain_file(ctx, keyfile)))
    return NULL;

  pass = password;
  SSL_CTX_set_default_passwd_cb(ctx, password_cb);
  if(!(SSL_CTX_use_PrivateKey_file(ctx, keyfile, SSL_FILETYPE_PEM)))
    return NULL;

  // Load the CAs we trust
  if(!(SSL_CTX_load_verify_locations(ctx, CA_LIST,0)))
    return NULL;

#if (OPENSSL_VERSION_NUMBER < 0x00905100L)
  SSL_CTX_set_verify_depth(ctx,1);
#endif

  return ctx;
}


/*!
** Load a Diffie Helman parameter in the context for key exchange
**
** @param ctx The SSL context
** @param file The Diffie Helman parameter file
**
** @return
*/
int load_dh_params(SSL_CTX *ctx, char *file)
{
  DH	*ret = 0;
  BIO	*bio;

  if ((bio = BIO_new_file(file,"r")) == NULL)
    return 0;

  ret = PEM_read_bio_DHparams(bio, NULL, NULL, NULL);
  BIO_free(bio);
  if(SSL_CTX_set_tmp_dh(ctx,ret) < 0)
    return 0;

  return 1;
}

/*!
** Destroy a SSL context
**
** @param ctx The SSL context
*/
void destroy_ctx(SSL_CTX *ctx)
{
  SSL_CTX_free(ctx);
}

