/*
** tlsfifo.h for tlsinject in
**
** Made by Averous Julien-Pierre
** E-Mail   <j.averous@sourcemac.com>
**
** Started on  Fri Dec  5 14:43:09 2008 Averous Julien-Pierre
** Last update Sat Dec  6 19:31:12 2008 Averous Julien-Pierre
*/

#ifndef TLSFIFO_H_
# define TLSFIFO_H_

# include <pthread.h>

/*
** Types
*/
typedef struct tls_fifo_item_s tls_fifo_item;
typedef struct tls_fifo_s tls_fifo;

typedef void (*fifo_free_item)(tls_fifo *, void *, unsigned);

/*
** Structures
*/
/// An internal object to hold an user data in the interna queue
struct tls_fifo_item_s
{
  void		*data;		/*!< The user data*/
  unsigned	data_len;	/*!< The user data size*/

  tls_fifo_item	*prev;		/*!< The previous item*/
  tls_fifo_item	*next;		/*!< The next item*/
};

/// The fifo object
struct tls_fifo_s
{
  pthread_mutex_t	mutex;	/*!< A mutex to protect out fifo linked array*/

  unsigned		total_size; /*!< The total amount of size stored*/
  tls_fifo_item		*first;	/*!< The first item of the queue*/
  tls_fifo_item		*last;	/*!< The last item of the queue*/

  fifo_free_item	free_foo; /*!< A user function to free an user data*/
};

/*
** Functions
*/
void tls_fifo_init(tls_fifo *fifo, fifo_free_item freer);
void tls_fifo_clean(tls_fifo *fifo);

void tls_fifo_push(tls_fifo *fifo, void *data, unsigned size);
int tls_fifo_pop(tls_fifo *fifo, void **data, unsigned *size);

unsigned tls_fifo_size(tls_fifo *fifo);

void tls_fifo_print(tls_fifo *fifo);

#endif /* !TLSFIFO_H_ */
